package SA::Network;
use strict;
use vars qw(@ISA $VERSION);

use SA;
use SA::MW;
use URI;
use IO::Socket;

@ISA = qw(SA::MW);
$VERSION = '1.0';

my @public_interfaces = qw(
		eth1
		ppp0
		ippp0
		);

=head1 NAME

SA::Network - configure network interfaces

=head1 SYNOPSIS

  use SA::Network;
  
  my $net = SA::Network->new();
  
  my $public = $net->get_ifprop('public');
  
  if ($public->{dhcpc_enabled})
  	{
  	$net->mod_if('public',
  			ipaddr  => '209.80.25.11',
  			netmask => '255.255.248.0',
  			dns1    => '209.80.25.108',
  			dns2    => '209.80.25.107',
  			);
  	}

=head1 DESCRIPTION

This module allows you to set the properties of any interface.  It may someday
contain specialized meta-methods to make specific operations easier.

=head1 METHODS

=over 4

=item new([CFG])

Returns an object instance against which the remaining methods may be called.
Provide the optional CFG (SA::Config object) or the constructor will get one of
its own from SA::MAIN_CONF. Provide the optional interface type of "public" to control whether to only restart the public interfaces on a commit.

=cut

sub new
{
	my $class = shift;
	my $cfg = shift;
	my $itype = shift || "";

	$class->SUPER::new($cfg, ('network/mod_config',$itype));
}

=item get_last_error()

Returns the SA::Error object corresponding to the last error.  Use this to find
out what happened when you get bad stuff back from one of the other functions.

=cut

=item set_hostname([HOSTNAME]), get_hostname()

Get and set global hostname.  Used for DNS, netbios name.  NOT FOR DHCP CLIENT
HOSTNAME; use the interface's c<dhcpc_hostname> property for that.

=cut

sub set_hostname
{
	my $self = shift;
	my $hostname = shift;

	# hostname can only contain alphanumerics and hyphens,
	# since it is also used as a dns resource record by BIND
	$hostname =~ /^[a-z0-9][a-z0-9-]*$/i or return
			$self->_set_error('network.invalid_hostname');

	# Make the hostname and domain name stay in sync to alleviate confusion
	# with email domains.
	my $domain = "$hostname.loc";
	$self->{_cfg}->set_leaf('network.domain', $domain);

	return $self->{_cfg}->set_leaf('network.hostname', $hostname);
}

sub get_hostname
{
	my $self = shift;
	return $self->{_cfg}->get_leaf('network.hostname');
}

=item get_publicif()

In scalar context, returns the device name of the current public interface.  In
list context, reutrns the device name and alias.

=cut

sub get_publicif
{
	my $self = shift;

	my $if = $self->{_cfg}->get_leaf('network.public_if');

	return wantarray() ? ($if, $self->aliaslookup($if)) : $if;
}

=item get_ifprop(IF)

IF must be a string containing either the name of an interface or its alias,
i.e. 'eth1' or 'public'.  This method returns a hash reference containing all
of the configuration parameters for the interface or undef if the interface
doesn't exist.

Do not assume that an interface has a static addr simply because it has
B<ipaddr> set.  The static addr settings (ipaddr, netmask, gateway, network,
bcast, dns1, dns2) will be set in the CFG each time the lease is renewed.  The
proper way to differentiate between a static and dynamic addr is to check
B<addr_type>.  This will be 'static' on a truly static interface.

=cut

sub get_ifprop
{
	my $self = shift;
	my $if = shift ||
			return $self->_set_error('network.invalid_if');

	# use more reliable function for 'public' lookup
	$if = $self->get_publicif()
		if $if eq 'public';

	# lookup the actual interface from the alias table
	$if = $self->iflookup($if) || $if;

	my $prop = $self->{_cfg}->get_tree("network.interfaces.$if") || {};

	# get calculated fields
	$prop->{if} = $if;
	$prop->{alias} = $self->aliaslookup($if);

	# for dynamic, get the info from file
	if ($prop->{addr_type} eq 'dynamic')
		{
		local $/ = undef; # slurp

		open INFO, "$SA_ROOT/conf/network/$if.info";
		my $eval_string = <INFO>;
		close INFO;

		my $VAR1; # keep us from bombing on eval under use strict
		my $info = eval "$eval_string" || {};

		# glue together the user-configurable properties and the dynamic
		# properties
		%$prop = (%$prop, %$info);
		}

	return $prop;
}

=item get_iflist(SORTFIELD)

Returns a sorted array reference of hash references containing the list of all
network interfaces and their configuration parameters.  Each entry will contain
the following keys:

	if - interface name
	alias - alias to interface name

See B<add_if> for descriptions of other keys based on interface type.

The default sort is on alias.

=cut

sub get_iflist
{
	my $self = shift;
	my $sortfield = shift || 'alias'; # default sort

	my @iflist = ();

	my $iftree = $self->{_cfg}->get_tree('network.interfaces') || {};
	
	foreach my $if (keys %$iftree)
		{
		push @iflist, {
				if       => $if,
				alias    => $self->aliaslookup($if), 
				%{ $iftree->{$if} },
				};
		}

	@iflist = sort { $a->{$sortfield} cmp $b->{$sortfield} } @iflist;

	return \@iflist
}

=item set_if(IF, HASH)

Irrevocably set the attributes of IF to HASH, removing what was there
previously.  HASH has elements as follows:

	addr_type - 'dynamic' or 'static' (required)
	
	DYNAMIC INTERFACE
	dhcpc_hostname

	STATIC INTERFACE
	ipaddr
	netmask
	gateway - will be calculated as the first address of the subnet if not provided
	bcast - will be calculated as the last address of the subnet if not provided
	dns1
	dns2

	PPP LOGIN SETTINGS (dialup or DSL with pppoe)
	phone - dialup only
	username
	password

	RUN DHCPD ON THIS INTERFACE?
	(this MAY be assumed to be true or false according to business rules that
	govern the behavior of each interface's alias)
	dhcpd_enabled - boolean
	dhcpd_start_range
	dhcpd_end_range

Returns false on failure.

=cut

sub set_if
{
	my $self = shift;
	my $if = shift ||
			return $self->_set_error('network.invalid_if');
	my %params = @_;

	return $self->_set_error('network.invalid_addr_type') unless $params{addr_type};

	# lookup the actual interface from the alias table
	$if = $self->iflookup($if, \%params) || $if;

	eval
		{
		if ($self->_check_all_params($if, \%params))
			{
			$self->{_cfg}->set_tree("network.interfaces.$if", \%params) or
					die $self->{_cfg}->get_last_error()->error();
			}
		};	

	return $self->_set_error($@) if $@;

	return $self->_unset_error();
}

=item mod_if(IF, HASH)

Same HASH parameters as add_if, returns false on failure.  You may not modify
an interface that doesn't exist.  Use add_if for that.

=cut

sub mod_if
{
	my $self = shift;
	my $if = shift ||
			return $self->_set_error('network.invalid_if');
	my %params = @_;

	# lookup the actual interface
	$if = $self->iflookup($if, \%params) || $if;

	eval
		{
		if ($self->_check_all_params($if, \%params))
			{
			foreach my $param (keys %params)
				{
				$self->{_cfg}->set_leaf("network.interfaces.$if.$param", $params{$param})
						or die $self->{_cfg}->get_last_error()->error();
				}
			}
		};

	return $self->_set_error($@) if $@;
	
	return $self->_unset_error();
}

=item commit()

Commits changes and fires C<network/mod_config> trigger.

=cut

############################
##### HELPER FUNCTIONS #####
############################
# The following are the "key" params:
# addr_type - indicates that mutually exclusive params need to be reset
# username - indicates user authentication validation is needed
# ipaddr - indicates static ipaddr validation is needed
# dhcpd_enabled - indicates that validation of start and end range is needed
# phone - indicates dialout validation is needed
# enabled - indicates that public_if needs to be checked
#
# resets mutually exclusive dynamic and static parameters
#
sub _check_addr_type
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	if ($params->{addr_type} eq 'dynamic')
		{
		# provide default
		$params->{dhcpc_hostname} = $params->{dhcpc_hostname} || 'myserver';

		$params->{ipaddr} = '';
		$params->{netmask} = '';
		$params->{netbits} = '';
		$params->{network} = '';
		$params->{gateway} = '';
		$params->{bcast} = '';
		}
	elsif ($params->{addr_type} eq 'static')
		{
		$params->{dhcpc_hostname} = '';
		}
	else
		{
		# not a good addr_type
		die 'network.invalid_addr_type';
		}

	return $self->_unset_error();
}

#
# validates that both username and password have been supplied
#
sub _check_username
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	die 'network.invalid_netlogon' unless $params->{password};

	return $self->_unset_error();
}

# these routines need Socket
use Socket;

#
# validates that all static ipaddr parameters belong to same subnet
# (provides default for missing bcast or gateway)
#
sub _check_ipaddr
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	# ipaddr, netmask must be a proper ipaddrs
	foreach ('ipaddr', 'netmask')
		{
		die 'network.invalid_ipaddr' unless inet_aton($params->{$_});
		}
	
	# get packed mask and network
	my $packed_mask = inet_aton($params->{netmask});
	my $packed_net = inet_aton($params->{ipaddr}) & $packed_mask;

	# detect collision between private and public addresses
	my $pub = $self->get_ifprop('public');
	if ($pub && ($if eq 'brg0'))
		{
		my $mask = inet_aton($pub->{netmask}) & $packed_mask;
		die 'network.invalid_ipaddr' if
			(inet_aton($pub->{ipaddr}) & $mask) eq
			(inet_aton($params->{ipaddr}) & $mask);
		}

	my $priv = $self->get_ifprop('private');
	if ($priv && ($self->aliaslookup($if) eq 'public'))
		{
		my $mask = inet_aton($priv->{netmask}) & $packed_mask;
		die 'network.invalid_ipaddr' if
			(inet_aton($priv->{ipaddr}) & $mask) eq
			(inet_aton($params->{ipaddr}) & $mask);
		}

	# bcast, if provided, needs to be in same network
	if ($params->{bcast})
		{
		die 'network.invalid_bcast' unless
				(inet_aton($params->{bcast}) & $packed_mask) eq $packed_net;
		}
	else
		{
		# generate - host part of max addr in subnet is xor of netmask and all 1's
		my $packed_maxaddr = $packed_mask ^ inet_aton('255.255.255.255');
		my $packed_bcast = $packed_net | $packed_maxaddr;
		$params->{bcast} = inet_ntoa($packed_bcast);
		}

	# gateway, if provided, needs to be in same network
	if ($params->{gateway})
		{
		die 'network.invalid_gateway' unless
				(inet_aton($params->{gateway}) & $packed_mask) eq $packed_net;
		}
	else
		{
		# Only regenerate the gateway if this is a public interface reconfig.
		# The gateway is a required input for the private side.
		if ($if ne "brg0")
			{
			# generate - host part of min addr in subnet is xor of netmask and 1
			my $packed_gw = $packed_net ^ inet_aton('0.0.0.1');
			$params->{gateway} = inet_ntoa($packed_gw);
			}
		}
	
	# always generate netbits and network
	$params->{netbits} = $self->netbits($packed_mask);
	$params->{network} = inet_ntoa($packed_net);
	
	return $self->_unset_error();
}

#
# validates that dhcpd_start_range and _end_range are on the proper subnet
# (and attached to a static addr interface)
#
sub _check_dhcpd_enabled
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	# all of this is moot if we don't have a static addr SOMEWHERE
	my $string_ipaddr = $params->{ipaddr} ||
			$self->{_cfg}->get_leaf("network.interfaces.$if.ipaddr");
	my $string_mask = $params->{netmask} ||
			$self->{_cfg}->get_leaf("network.interfaces.$if.netmask");
	die 'network.invalid_ipaddr' unless
			($string_ipaddr && $string_mask);

	# Only check the start and end range if dhcpd is enabled
	if ($params->{dhcpd_enabled}){
		# get packed mask and network
		my $packed_mask = inet_aton($string_mask);
		my $packed_net = inet_aton($string_ipaddr) & $packed_mask;

		# start_range must be on subnet
		die 'network.invalid_start_range' unless
			(inet_aton($params->{dhcpd_start_range}) & $packed_mask) eq $packed_net;

		# end_range must be on subnet
		die 'network.invalid_end_range' unless
			(inet_aton($params->{dhcpd_end_range}) & $packed_mask) eq $packed_net;
		# start_range must be higher than end range
		die 'network.invalid_start_range' unless
			(inet_aton($params->{dhcpd_start_range}) lt inet_aton($params->{dhcpd_end_range}));

		# Server IP cannot be in DHCP range
		die 'network.invalid_ipaddr' unless
			((inet_aton($string_ipaddr) lt inet_aton($params->{dhcpd_start_range})) ||
			(inet_aton($string_ipaddr) gt inet_aton($params->{dhcpd_end_range})))
;
	}

	return $self->_unset_error();
}

#
# validates phone number
#
#sub _check_phone
#{
#	my $self = shift;
#	my $if = shift;
#	my $params = shift;
#
#	die 'network.invalid_phone' unless
#			$params->{phone} =~ /
#			[^\d]?    # optional non-digit
#			(\d{3})?  # optional areacode
#			[^\d]?    # optional non-digit
#			\d{3}     # three-digit prefix
#			[^\d]?    # optional non-digit
#			\d{4}     # four-digit number
#			/x;
#
#	return $self->_unset_error();
#}

sub _check_enabled
{
	my $self = shift;
	my $if = shift;
	my $params = shift;

	# we only do special stuff for public interfaces changing enabled status
	if ($self->aliaslookup($if) eq 'public')
		{
		if ($params->{enabled})
			{
			# enabled has been set to a true value
			# I have to reset all the other public
			$self->_reset_public();
			$self->{_cfg}->set_leaf('network.public_if', $if);
			}
		elsif (defined $params->{enabled} && ! $params->{enabled})
			{
			# enabled has been set to a false value
			# I have to unset public_if
			$self->{_cfg}->rm_leaf('network.public_if');
			}

		# enabled has not been provided: noop
		}

	return $self->_unset_error();
}

sub _reset_public
{
	my $self = shift;

	foreach my $if (@public_interfaces)
		{
		$self->{_cfg}->set_leaf("network.interfaces.$if.enabled", 0);
		}
	
	return 1;
}

#
# return netbits given packed ('CCCC') netmask
#
sub netbits
{
	my $self = shift;
	my $packed_mask = shift || return 0;

	my $int_mask = unpack('I', $packed_mask);

	my $bitcount = 0;

	for (my $i = 0; $i < 32; $i++)
		{
		my $bitmask = (1 << $i); # walk a one across a 32-bit int
		$bitcount += ($int_mask & $bitmask) ? 1 : 0;
		}

	return $bitcount;
}

sub iflookup
{
	my $self = shift;
	my $alias = shift;
	my $params = shift;

	# be careful with this; it is trickle-down logic
	return 'brg0' if $alias eq 'private';
	return 'ippp0' if ($alias eq 'public' && $params->{username} && $params->{msn});
	return 'ppp0' if ($alias eq 'public' && $params->{username});
	return 'eth1' if $alias eq 'public';
	return '';
}

sub aliaslookup
{
	my $self = shift;
	my $if = shift;

	# be careful with this; it is trickle-down logic
	return 'public' if ($if eq 'ppp0' || $if eq 'eth1' || $if eq 'ippp0');
	return 'private' if $if eq 'brg0';
	return '';
}

sub get_macaddr
{
	my $self = shift;
	my $if = shift;

	local $/ = undef; # this stays in scope during sys_std_out()

	my $output = sys_std_out("/sbin/ifconfig $if");

	# parse out the mac address
	my ($mac_addr) = $output =~ /HWaddr ([0-9A-F:]+)/;

	return $mac_addr;
}

=item connect(URL, [TIMEOUT])

Requires URL.  Will return true if URL allows connections to port 80 (http)
within TIMEOUT seconds (or will try for 60 seconds if TIMEOUT is false).  Will
return false if the unit is not configured for a public internet connection or
the site doesn't allow connections to port 80 (http).

=cut

sub connect
{
	my $self = shift;
	my $url = shift || return
			$self->_set_error('network.invalid_url');
	my $timeout = shift || 60;

	my $success = '';

	# return false immediately if I'm not configured
	$self->get_publicif() or
			return $self->_set_error('network.not_configured');

	# parse hostname from URL
	my $uri = URI->new($url);
	my $host = $uri->host() or return
			$self->_set_error('network.invalid_host');

	# verify connection to port 80 or try to wake the dead
	local $SIG{ALRM} = sub { die };
	eval
		{
		alarm $timeout;
		while (1)
			{
			$success = IO::Socket::INET->new("$host:80");
			last if $success;
			sleep 1;
			}
		};

	alarm 0;

	return $success ? 1 : 0;
}

1;

__END__

=back

=head1 ERRORS

=over 4

=item network.invalid_if

An invalid interface was provided (probably blank).

=item network.if_no_exist

A method requiring an interface that already exists was called with a non-exisitent interface or alias.

=item network.if_exists

A method requiring that an interface not exist was called with an already existent interface name or alias.

=item network.invalid_ipaddr

An invalid ip address or netmask was provided.

=item network.invalid_bcast

A broadcast address was provided that is not on the same subnet as the ip address.

=item network.invalid_gateway

A gateway address was provided that is not on the same subnet as the ip address.

=item network.invalid_start_range

A start range address was provided that is not on the same subnet as the ip address.

=item network.invalid_end_range

An end range address was provided that is not on the same subnet as the ip address.

=item network.invalid_phone

An invalid phone number was provided.

=item network.invalid_netlogon

An invalid username or password was provided.

=item network.invalid_addr_type

An invalid address type was provided (should be dynamic or static ONLY).

=item network.wait_timed_out

The timeout period provided to the commit method was exceeded and the transaction was rolled back.

=item OTHER ERRORS

Any errors thrown by SA::Config or SA::Exec.

=back

=head1 AUTHOR

Chad T. Carr <ccarr@franzdoodle.com>

=head1 SEE ALSO

perl(1), SA::Config(3), SA::Exec(3).

=cut
