package SA::Network::StaticIP;
use strict;
use vars qw(@ISA);

use SA;
use SA::Network;
use SA::MW;

@ISA = qw(SA::MW);

=head1 NAME

SA::Network::StaticIP - Allocate and keep track of reserved static IP addresses

=head1 SYNOPSIS


=head1 DESCRIPTION

Allocates and deallocates reserved static IP addresses at the end of the DHCP range or subnet range.

=head1 METHODS

=over 4

=item new([CFG])

Pass an SA::Config object or it will instance one for you.  Returns undef on failure.

=cut

sub new
{
	my ($class, $cfg) = @_;

	my $self = $class->SUPER::new($cfg, 'dummy') or
			return $class->_set_error($class->SUPER::get_last_error());

	$self->{_staticips} = SA::Config->new("$SA_ROOT/conf/staticip.conf");

	return $self;
}

=item get_nextfreeip([COUNT])

Returns an array of hashes which contains a list of allocated static ips.

=cut

sub get_nextfreeip
{
	my $self = shift;
	my $count = shift;

	my @staticiplist = ();

	my $currentlist = $self->{_staticips}->get_tree('ip');

	for (my $i = 0; $i < $count; $i++) {
		foreach my $ipnum (keys %{ $currentlist })
		{
			my $ipentry = $self->{_staticips}->get_tree("ip.$ipnum");
			if ($ipentry->{used}) {
				next;
			} else {
				$currentlist->{$ipnum}{used} = 1; 
				push @staticiplist, $ipentry;
				last;
			}
		}
	}
	$self->{_staticips}->set_tree('ip',$currentlist);
	$self->{_staticips}->commit();

	return @staticiplist;
}

=item set_ipentry(ARRAY)

Sets each matching record in the static IP database to the values passed in. This function can be used to free an IP entry by setting the "used" field to 0.

=cut

sub set_ipentry
{
	my $self = shift;
	my $ip_array = shift;

	my $len = scalar(@$ip_array);

	my $currentlist = $self->{_staticips}->get_tree('ip');
	
	for (my $i=0;$i < $len; $i++) {
		my $ipentry = shift(@$ip_array);
		foreach my $ipnum (keys %{ $currentlist })
		{
			my $curentry = $self->{_staticips}->get_tree("ip.$ipnum");
			if ($ipentry->{addr} eq $curentry->{addr}) {
				$currentlist->{$ipnum} = $ipentry; 
				last;
			} else {
				next;
			}
		}
	}

	$self->{_staticips}->set_tree('ip',$currentlist);
	$self->{_staticips}->commit();
}

=item InitStaticList()

Initializes the static IP address database.  This should be called any time the private IP address is changed on the system.

=cut

sub InitStaticList
{
	my $self = shift;

	my $network = SA::Network->new({read_only=>1});

	my $priv = $network->get_ifprop('private');
	my $reserved = $priv->{static_reserve} || 1;
	
	# default to the end of the static range or the end of the
	# subnet (bcast)
	my $static_end = $priv->{static_end} || $priv->{bcast};

	my ($dum1,$dum2,$dum3,$end_addr) = split '\.', $static_end;
	my $start_addr = $end_addr - $reserved - 1; #Leave space for dialin
	if ($start_addr <= 2) {
		$start_addr = 2;
	}

	open STATIC_CONF, "> /sa2/conf/staticip.conf";
	my $count = $end_addr - $start_addr - 1; #Leave space for dialin
	my $addr = $start_addr;
	for (my $i=1; $i <= $count; $i++) {
		print STATIC_CONF "ip.$i.used=0\n";
		print STATIC_CONF "ip.$i.addr=$dum1.$dum2.$dum3.$addr\n";
		print "ip.$i.addr=$dum1.$dum2.$dum3.$addr\n";
		print "ip.$i.used=0\n";
		$addr++;
	}
	close STATIC_CONF;
	system("/bin/sync");
}

=item commit()

Writes changes to CFG.

=cut

1;

__END__

=head1 AUTHOR

Brian Raccuglia (Brian.Raccuglia@tais.com)

=head1 SEE ALSO

perl(1), SA::MW(3).

=cut
